/*
Fusion Comp Exporter for Premiere Pro v0.2
2018/x/x    - First version
2020/02/19  - Updated - Added support for sped up and slowed down clips (not time remapped).
2020/03/25  - Updated - Added supports for time reversed clips.

Notes:
    - Does not export R3D RAW files' data.
    - If you have changed speed in the clip and export with handles, time speed node in fusion does not show correct range.

Created by Arttu Rautio (aturtur)
Webpage http://aturtur.com
*/

$.runScript = {
    exportComp: function() {
        app.enableQE(); // Enable QE Dom
        // ----------------------------------------------------------------------------------------------------------------
        // Functions
        var splitByLast = function(text, separator) {
            // Split string by last found separator
            var index = text.lastIndexOf(separator);
            return [text.slice(0, index), text.slice(index + 1)]
        }

        var tickToFps = function(tick) {
            // Premiere's Tick format converted to Frame Per Second format
            switch(tick) {
                var fps = 0;
                case 25401600000: fps = 10;     break;
                case 21168000000: fps = 12;     break;
                case 20321280000: fps = 12.5;   break;
                case 16934400000: fps = 15;     break;
                case 10594584000: fps = 23.973; break;
                case 10584000000: fps = 24;     break;
                case 10160640000: fps = 25;     break;
                case 8475667200:  fps = 29.97;  break;
                case 8467200000:  fps = 30;     break;
                case 5080320000:  fps = 50;     break;
                case 4237833600:  fps = 59.94;  break;
                case 4233600000:  fps = 60;     break;
                default:          fps = 25;
                return fps;
            }
        }
        // ----------------------------------------------------------------------------------------------------------------
        // Common variables
        var seq = app.project.activeSequence;       // Get active sequence
        var seqName = seq.name;                     // Get sequence's name
        var resHeight = seq.frameSizeVertical;      // Get sequence's height resolution
        var resWidth = seq.frameSizeHorizontal;     // Get sequence's width resolution
        var trackCount = seq.videoTracks.numTracks; // Get video track count
        var tick = seq.timebase;                    // Get sequence's tick
        var fps = tickToFps(tick);                  // Convert tick to fps
        
        // Initialize variables for later use
        var loaders = [];                           // Initialize list for loader nodes
        var timespeeds = [];                        // Initialize list for time speed nodes
        var loaderCount = 0;                        // Initialize loader count variable
        var fileFormat = "";                        // Initialize string variable for file format type
        var error = 0;                              // Initialize variable for error
        var atLeastOne = 0;                         //
        // ----------------------------------------------------------------------------------------------------------------
        // Run the main thing
        for (var k = 0; k< trackCount; k++) {       // Iterate through video tracks
            var track = seq.videoTracks[k];         // Get video track
            var clipCount = track.clips.numItems;   // Get clip count
            for (var i = 0; i < clipCount; i++) {   // Iterate through video clips
                var clip = track.clips[i];          // Get video clip
                if (clip.isSelected() == true) {    // If video clip is selected
                    atLeastOne = 1;                 // At least one selected video clip
                    var metadata = clip.projectItem.getProjectMetadata(); // Get clip's metadata
                    var split = metadata.split("MediaDuration>")[1];
                    var duration = split.split("<")[0];                                  // Get duration
                    var HH = parseInt(duration.split(":")[0]);                           // Hours
                    var MM = parseInt(duration.split(":")[1]);                           // Minutes
                    var SS = parseInt(duration.split(":")[2]);                           // Seconds
                    var FF = parseInt(duration.split(":")[3]);                           // Frames
                    var fileName = clip.projectItem.getMediaPath();                      // Get file name
                    fileName = fileName.replace(/\\/g , "\\\\");                         // For Windows paths
                    fileName = "\""+fileName+"\"";                                       // 
                    var reversed = clip.isSpeedReversed()                                // Check is video clip's speed reversed
                    var speed = clip.getSpeed()                                          // Get video clip's speed multiplier
                    var length = ((HH*3600)*fps)+((MM*60)*fps)+(SS*fps)+FF;              // Calculate clip's length
                    var trimIn = Math.floor((clip.inPoint.ticks / tick) * speed);        // Calculate clip's in point
                    var trimOut= Math.floor(((clip.outPoint.ticks / tick) - 1) * speed); // Calculate clip's out point
                    var fileExtension = splitByLast(fileName,".")[1];                    //
                    fileExtension = fileExtension.toLowerCase();                         //
                    fileExtension = fileExtension.replace(/[^\w\s]/gi, '')               // Get file extension
                    var startFrame = splitByLast(fileName,"_")[1];                       //
                    startFrame = splitByLast(startFrame,".")[0];                         // Get start frame
                    if (reversed == true) {                                              // If clip is time reversed
                        oldTrimIn = trimIn;                                              //
                        trimIn = length - trimOut - 1;                                   // Calculate time reversed in point
                        trimOut = length - oldTrimIn - 1;                                   // Calculate time reversed out point
                    }
                    // *************************************************************************************************
                    // Check if there is room for handles
                    if (trimIn - handles < 0) {
                        alert("No enough space for handles!\nClip: " + fileName);
                        error = 1;
                    } else if (length - trimOut + handles > length) {
                        alert("No enough space for handles!\nClip: " + fileName);
                        error = 1;
                    }
                    // *************************************************************************************************
                    // File format stuff
                    switch(fileExtension){
                        case "r3d":
                            var formatId = "\"R3DFormat\"";
                            var sf = "";
                            var formatSpecific = "[\"Clip1.R3DFormat.ColorScience\"] = Input { Value = 1, },";
                            break;
                        case "mov":
                            var formatId = "\"QuickTimeMovies\"";
                            var sf = "";
                            var formatSpecific = "";
                            break;
                        case "mp4":
                            var formatId = "\"QuickTimeMovies\"";
                            var sf = "";
                            var formatSpecific = "";
                            break;
                        case "exr":
                            var formatId = "\"OpenEXRFormat\"";
                            var sf = "StartFrame = "+startFrame+",";
                            var formatSpecific = "[\"Clip1.OpenEXRFormat.RedName\"] = Input { Value = FuID { \"R\" }, },\
                                    [\"Clip1.OpenEXRFormat.GreenName\"] = Input { Value = FuID { \"G\" }, },\
                                    [\"Clip1.OpenEXRFormat.BlueName\"] = Input { Value = FuID { \"B\" }, },";
                            break;
                        case "dpx":
                            var formatId = "\"DPXFormat\"";
                            var sf = "StartFrame = "+startFrame+",";
                            var formatSpecific = "[\"Clip1.DPXFormat.SLogVersion\"] = Input { Value = FuID { \"SLog2\" }, },";
                            break;
                        case "png":
                            var formatId = "\"PNGFormat\"";
                            var sf = "StartFrame = "+startFrame+",";
                            var formatSpecific = "";
                            break;
                        case "jpg":
                            var formatId = "\"JpegFormat\"";
                            var sf = "StartFrame = "+startFrame+",";
                            var formatSpecific = "";
                            break;
                        case "tif":
                            var formatId = "\"TiffFormat\"";
                            var sf = "StartFrame = "+startFrame+",";
                            var formatSpecific = "";
                            break;
                        default:
                            break;
                    }
                    var loaderid = loaderCount+parseInt(1);             // Get current loader ID
                    var loaderypos = loaderCount*parseInt(50);          // Get y-position for current loader
                    var loader = "\n\t\tLoader"+loaderid+" = Loader {\
                 Clips = {\
                    Clip {\
                        ID = \"Clip1\",\
                        Filename = "+fileName+",\
                        FormatID = "+formatId+",\
                        "+sf+"\
                        Length = "+length+",\
                        LengthSetManually = true,\
                        TrimIn = "+(trimIn-handles)+",\
                        TrimOut = "+(trimOut+handles)+",\
                        ExtendFirst = 0,\
                        ExtendLast = 0,\
                        Loop = 1,\
                        AspectMode = 0,\
                        Depth = 0,\
                        TimeCode = 0,\
                        GlobalStart = "+globalStart+",\
                        GlobalEnd = "+(((trimOut+handles)-(trimIn-handles))+globalStart)+"\
                        }\
                        },\
                        CtrlWZoom = false,\
                        Inputs = {\
                                [\"Gamut.SLogVersion\"] = Input { Value = FuID { \"SLog2\" }, },\
                                "+formatSpecific+"\
                            },\
                        ViewInfo = OperatorInfo { Pos = { 100, "+loaderypos+" } },\
                        }";
                    // *************************************************************************************************
                    // Create Time Speed node if necessary
                    if (reversed == true) { // If time reversed
                        speed = speed * -1.0
                    }
                    if (speed != 1) { // If speed is not 100%
                        var timespeed = "\n\t\tTimeSpeed"+loaderid+" = TimeSpeed {\
            Inputs = {\
                Speed = Input { Value = "+speed+", },\
                Input = Input {\
                    SourceOp = \"Loader"+loaderid+"\",\
                    Source = \"Output\",\
                },\
            },\
            ViewInfo = OperatorInfo { Pos = { 250, "+loaderypos+" } },\
        }";
                        timespeeds.push(timespeed); // Add time speed to time speeds list
                    }
                    // *************************************************************************************************
                    loaders.push(loader); // Add loader to loaders list
                    loaderCount++; // Increase loader count
                } else {
                    // Do nothing if clip is not selected
                }
            }
        }
        // ----------------------------------------------------------------------------------------------------------------
        // Initialize Fusion Composition Begin
        var compStart = "Composition {\
    CurrentTime = 0,\
    RenderRange = { "+globalStart+", "+(globalStart+1000)+" },\
    GlobalRange = { "+globalStart+", "+(globalStart+1000)+" },\
    CurrentID = "+(loaderCount+1)+",\
    PlaybackUpdateMode = 0,\
    Version = \"Fusion 9.0.2 build 15\",\
    SavedOutputs = 2,\
    HeldTools = 0,\
    DisabledTools = 0,\
    LockedTools = 0,\
    AudioOffset = 0,\
    AutoRenderRange = true,\
    Resumable = true,\
    OutputClips = {\
    },\
    Tools = {";
        
        // Initialize Fusion Composition Ending
        var compEnd = "\n\t},\
    Prefs = {\
        Comp = {\
            Views = {\
                Right = {\
                },\
                Left = {\
                    Viewer = {\
                    },\
                },\
                View1 = {\
                },\
            },\
            FrameFormat = {\
                Rate = "+fps+",\
                GuideRatio = 1.77777777777778,\
                DepthFull = 2,\
                DepthPreview = 2,\
                DepthInteractive = 2,\
            },\
            Unsorted = {\
                GlobalEnd = 100\
            },\
            Paths = {\
            },\
            FlowView = {\
                GridSnap = true,\
            },\
            QuickTime = {\
            },\
        }\
    }\
}";
        // ----------------------------------------------------------------------------------------------------------------
        // Finalize Fusion Composition File        
        loaders = loaders.join(",");               // Combine loaders        
        if (timespeeds.length > 0) {               // If there is any time speed nodes
            timespeeds = ","+timespeeds.join(","); // Combine time speed nodes
        } else {                                   // Otherwise
            timespeeds = "";                       // Return empty string
        }
        
        // Combine final composition string
        var comp = compStart + loaders + timespeeds + compEnd;

        // Exporting
        if (atLeastOne == 1){                                    // At least one loader
            if (error == 0) {                                    // If no error
                var theFile = File.saveDialog("Where to save?",
                "Fusion Composition:*.comp");                    // Save dialog
                if (theFile != null) {                           // If user didn't cancel save dialog
                    theFile.open("w","TEXT","????");             // Open file for writing
                            theFile.encoding = "UTF-8";          // Set encoding
                            theFile.writeln(comp);               // Write comp
                    theFile.close();                             // Close file
                    alert(loaderCount+" clips exported!");       // Alert how many clips exported
                    if (execute == true) {                       // If "Open Comp" checked
                        theFile.execute();                       // Execute saved comp file
                    }
                }
            }
        }

    }
}